@@ -6,7 +6,7 @@
  * Provides the client half of the process.  Should be considered
  * untrusted by the privman server.
  *
- * $Id: priv_client.cc,v 1.35 2002/10/31 16:11:39 dougk Exp $
+ * $Id: priv_client.cc,v 1.38 2002/11/12 04:43:50 dougk Exp $
  */
 
 #include "../config.h"
@@ -247,11 +247,18 @@ void priv_sep_init(void (*servfn)(void),
 
 void priv_init(const char *appname)
 {
+    /* Syslog init. */
+    openlog("privman", LOG_PID, LOG_AUTHPRIV);
+
     /* Read the config now. */
     readConfig(appname);
 
     priv_sep_init(privman_serv_init, 0, 0,
             config->unpriv_user.c_str(), config->unpriv_jail.c_str());
+
+    /* If in the child, close the syslog() fd */
+    if (child_pid > 0)
+        closelog();
 }
 
 
@@ -554,10 +561,10 @@ int priv_pam_set_item(pam_handle_t *pamh, int item_type, const void *item)
     }
 
     /* send the message */
-    msg_sendmsg(msg, privmand_fd, "priv_pam_get_item(sendmsg)");
+    msg_sendmsg(msg, privmand_fd, "priv_pam_set_item(sendmsg)");
 
     msg_clear(msg);
-    msg_recvmsg(msg, privmand_fd, "priv_pam_get_item(recvmsg)");
+    msg_recvmsg(msg, privmand_fd, "priv_pam_set_item(recvmsg)");
 
     rc = msg_getInt(msg); /* were we denied at the gate? */
     if (rc < 0) {
@@ -567,7 +574,7 @@ int priv_pam_set_item(pam_handle_t *pamh, int item_type, const void *item)
 
     assert((enum privman_responces)rc == PRIV_PAM_RC);
 
-    rc = msg_getInt(msg); /* and the RC of pam_get_item. */
+    rc = msg_getInt(msg); /* and the RC of pam_set_item. */
 
     /* Wait until success to set the cache. */
     if (rc == PAM_SUCCESS) {
@@ -1057,11 +1064,16 @@ FILE*   priv_popen_as(const char *command, const char *type, const char *user)
 
     pid = priv_rerunas(priv_popen_impl, args, user, NULL,
             PRIV_RR_OLD_SLAVE_MONITORED);
+    if (pid < 0) {
+        close(sock);
+        unlink(socketname);
+        return NULL;
+    }
 
     /* 3.  Get fd.  Close off socket since we are not listening anymore. */
     int         fd;
     if ((fd = accept(sock, 0, 0)) < 0) {
-        perror("accept failed.\n");
+        perror("accept failed");
         boom("priv_popen_as(accept)");
     }
 
@@ -1112,7 +1124,7 @@ int priv_register_info_fn(char *(*fnptr)(char * const *))
     int handle;
  
     /* Must be done before the call to priv_init() */
-    if (child_pid == 0) {
+    if (geteuid() != 0) {
         errno = EPERM;
         return -1;
     }
@@ -1128,7 +1140,7 @@ int priv_register_cap_fn(int (*fnptr)(char * const *))
     int handle;
  
     /* Must be done before the call to priv_init() */
-    if (child_pid == 0) {
+    if (geteuid() != 0) {
         errno = EPERM;
         return -1;
     }
@@ -1173,7 +1185,7 @@ int priv_invoke_cap_fn(int handle, char * const args[])
     message_t  *msg = msg_new();
     int         rc;
 
-    msg_init(msg, CMD_CUSTOM_INFO);
+    msg_init(msg, CMD_CUSTOM_CAP);
 
     msg_addInt(msg, handle);
     msg_addArgv(msg, args);
