@@ -6,7 +6,7 @@
  * Header file for the implimentation bits of privman.  Things common
  * between the server and the client halves.
  *
- * $Id: priv_impl.h,v 1.20 2002/10/30 17:34:06 dougk Exp $
+ * $Id: priv_impl.h,v 1.21 2002/11/12 04:43:51 dougk Exp $
  */
 
 
@@ -114,10 +114,10 @@ void msg_sendmsg(message_t *msg, int fd, const char *boommsg) {
 static __inline__
 void msg_addArgv(message_t *msg, char * const argv[]) {
     int i;
-    for (i = 0; argv[i] != NULL; ++i)
+    for (i = 0; argv != NULL && argv[i] != NULL; ++i)
         ;
     msg_addInt(msg, i);
-    for (i = 0; argv[i] != NULL; ++i)
+    for (i = 0; argv != NULL && argv[i] != NULL; ++i)
         msg_addString(msg, argv[i]);
 }
 
