@@ -6,7 +6,7 @@
  * Provides the server half of the process.  This half retains priviledge
  * and should be nei-invulnerable.
  *
- * $Id: privman.cc,v 1.50 2002/10/31 16:11:39 dougk Exp $
+ * $Id: privman.cc,v 1.54 2002/11/12 04:43:51 dougk Exp $
  */
 #include "../config.h"
 #include <stdio.h>
@@ -47,6 +47,13 @@
 
 #include "types.h"
 #include "priv_impl.h"
+
+#ifdef __cplusplus
+#define UNUSED(p)
+#else
+#define UNUSED(p) p __attribute((unused))
+#endif
+
 /* Globals */
 int             privmand_fd     = -1;    /* FD to talk to the other process */
 
@@ -141,12 +148,22 @@ static bool openPerm(const char *path, accessType_t type)
     return true;
 }
 
+/* True is "user" is mentioned in a runas statement.
+ * Or if '*' is mentioned, and user is not root.
+ */
 static bool runasPerm(const char *user)
 {
-    if (user[0] != '\0' && config->user.count(user) == 0
-            && config->user.count("*") == 0)
+    if (user == NULL || user[0] == '\0' || !strcmp(user, "*"))
         return false;
-    return true;
+    if (config->user.count(user) > 0)
+        return true;
+    if (config->user.count("*") > 0) {
+        struct passwd *pw = getpwnam(user);
+        if (pw == NULL || pw->pw_uid == 0)
+            return false;
+        return true;
+    }
+    return false;
 }
 
 static void sendEPERM(message_t *msg, const char *reason)
@@ -371,12 +388,6 @@ static       struct pam_conv    pconv;    /* server side. */
  * side has it.  resp is an out value, not an in value.
  */
 
-#ifdef __cplusplus
-#define UNUSED(p)
-#else
-#define UNUSED(p) p __attribute((unused))
-#endif
-
 static int convert_punt(int num_msg,
         PAM_CONV_FUNC_CONST struct pam_message **messages,
         struct pam_response **resp,
@@ -434,6 +445,31 @@ static void pamStart(message_t *msg)
     free(service); free(user);
 }
 
+static void pamAuthenticate(message_t *msg)
+{
+    pam_handle_t *pamh;
+    int flags, rc;
+
+    pamh  = (pam_handle_t*)msg_getPtr(msg);
+    flags = msg_getInt(msg);
+
+    rc = pam_authenticate(pamh, flags);
+
+    if (rc == PAM_SUCCESS && config->auth_allow_rerun) {
+        char *authenticating_user;
+        int rc2;
+        rc2 = pam_get_item(pamh, PAM_USER,
+                (PAM_GET_ITEM_CONST void**)(&authenticating_user));
+        if (rc2 == PAM_SUCCESS)
+            config->user.insert(authenticating_user);
+    }
+
+    /* This isn't a call to run the conv func */
+    msg_initResponce(msg, PRIV_PAM_RC);
+    msg_addInt(msg, rc);
+    msg_sendmsg(msg, privmand_fd, "pamSimpleFunc(sendmsg)");
+}
+
 static void pamSimpleFunc(message_t *msg, int (*func)(pam_handle_t*,int))
 {
     pam_handle_t *pamh;
@@ -642,7 +678,7 @@ static void rerunAsProcess(message_t *msg)
 
     /* MAC checks. */
     if (!runasPerm(user)) {
-        sendEPERM(msg, "Unauthorized respawnAs");
+        sendEPERM(msg, "Unauthorized rerunAs target");
         goto cleanup;
     }
 
@@ -690,7 +726,7 @@ static void respawnAsProcess(message_t *msg)
 
     /* MAC checks. */
     if (!runasPerm(user)) {
-        sendEPERM(msg, "Unauthorized respawnAs");
+        sendEPERM(msg, "Unauthorized respawnAs target");
         goto clean;
     }
 
@@ -734,7 +770,7 @@ static void customInfo(message_t *msg)
 {
     int         handle  = msg_getInt(msg);
     char      **args    = msg_getArgv(msg);
-    char       *rv      = NULL;
+    char       *rv;
     info_fn_map_t::iterator    it;
 
     it = info_fn_map.find(handle);
@@ -742,6 +778,7 @@ static void customInfo(message_t *msg)
         rv = ((*it).second)(args);
     } else {
         errno = ENOENT;
+        rv = NULL;
     }
 
     msg_initResponce(msg, PRIV_PAM_RC);
@@ -769,7 +806,7 @@ static void customCap(message_t *msg)
     if (it != cap_fn_map.end()) {
         rv = ((*it).second)(args);
     } else {
-        rv = 0;
+        rv = -1;
         errno = ENOENT;
     }
 
@@ -843,7 +880,7 @@ static void control_loop(void) {
         c = (enum commands)msg_getInt(msg);
 
         if (!validRequest(c)) {
-            sendEPERM(msg, "Unknown request");
+            sendEPERM(msg, "Unknown or not permitted request");
             continue;
         }
         /* The handler function for a given request does the responce. */
@@ -869,7 +906,6 @@ static void name(message_t *msg) {                                      \
     pamSimpleFunc(msg, method);                                         \
 }
 
-PAM_SIMPLE_HANDLER(pamAuthenticate,     pam_authenticate)
 PAM_SIMPLE_HANDLER(pamAcctMgmt,         pam_acct_mgmt)
 PAM_SIMPLE_HANDLER(pamEnd,              pam_end)
 PAM_SIMPLE_HANDLER(pamSetcred,          pam_setcred)
@@ -914,12 +950,7 @@ void privman_serv_init(void)
     function_map[ CMD_CUSTOM_INFO       ] = customInfo;
     function_map[ CMD_CUSTOM_CAP        ] = customCap;
 
-
-    /* Syslog init. */
-    openlog("privman", LOG_PID, LOG_AUTHPRIV);
-
-
-    /* And the server spends most of its time in here. */
+    /* The server spends most of its time in here. */
     control_loop();
 
     /* Two ways we get here
