@@ -9,7 +9,7 @@
  * For privman test purposes, its uses the "login" pam stack instead
  * of its own.
  *
- * $Id: check_user.c,v 1.7 2002/09/16 20:13:17 dougk Exp $
+ * $Id: check_user.c,v 1.9 2002/11/12 23:26:27 dougk Exp $
  */
 
 #include "../config.h"
@@ -23,11 +23,31 @@
 #include <pam/pam_misc.h>
 #endif
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 static struct pam_conv conv = {
     misc_conv,
     NULL
 };
 
+void rerunfn(char * const args[])
+{
+    struct passwd *pw = getpwuid(getuid());
+    if (pw != NULL)
+        fprintf(stdout, "rerun as %s(%d)\n", pw->pw_name, pw->pw_uid);
+    else
+        fprintf(stdout, "rerun as %d\n", getuid());
+    exit(0);
+}
+
 int main(int argc, char *argv[])
 {
     pam_handle_t *pamh=NULL;
@@ -50,29 +70,36 @@ int main(int argc, char *argv[])
 
     if (retval != PAM_SUCCESS) {
         fprintf(stdout, "pam_start failed.\n");
-        goto failed;
+        goto finished;
     }
         
     retval = priv_pam_authenticate(pamh, 0); /* is user really user? */
     if (retval != PAM_SUCCESS) {
         fprintf(stdout, "Not Authenticated!\npam_authenticate failed.\n");
-        goto failed;
+        goto finished;
     }
 
     retval = priv_pam_acct_mgmt(pamh, 0);    /* permitted access? */
     if (retval != PAM_SUCCESS) {
         fprintf(stdout, "pam_acct_mgmt failed.\n");
-        goto failed;
+        goto finished;
     } else {
         fprintf(stdout, "Authenticated\n");
     }
 
-failed:
+finished:
     if (priv_pam_end(pamh,retval) != PAM_SUCCESS) {     /* close Linux-PAM */
         pamh = NULL;
         fprintf(stderr, "check_user: failed to release authenticator\n");
         exit(1);
     }
 
-    return ( retval == PAM_SUCCESS ? 0:1 );       /* indicate success */
+    if (retval == PAM_SUCCESS) {
+        retval = priv_rerunas(rerunfn, 0, user, NULL, 0);
+        if (retval < 0)
+            perror("priv_rerunas");
+        _exit(0);
+    } else {
+        return 1;
+    }
 }
